Package: DSAIRM
Type: Package
Title: Dynamical Systems Approach to Immune Response Modeling
Description: Simulation models (apps) of various within-host immune response scenarios.
    The purpose of the package is to help individuals learn 
    about within-host infection and immune response modeling from a dynamical systems perspective.
    All apps include explanations of the underlying models and instructions on what to do with the models. 
Version: 0.9.6
Date: 2023-08-23
Authors@R: c(
    person("Andreas", "Handel", email = "andreas.handel@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-4622-1146") ),
    person("Cody", "Dailey",  role = "ctb"), 
    person("Yang", "Ge",  role = "ctb"),
    person("Spencer", "Hall",  role = "ctb"),
    person("Brian", "McKay",  role = "ctb"),
    person("Sina", "Solaimanpour",  role = "ctb"),
    person("Alexis", "Vittengl",  role = "ctb"),
    person("Henok", "Woldu",  role = "ctb") 
    )
Maintainer: Andreas Handel <andreas.handel@gmail.com>
License: GPL-3
Encoding: UTF-8
LazyData: TRUE
Imports: adaptivetau, boot, deSolve, dplyr, ggplot2, gridExtra, lhs,
        nloptr, plotly, rlang, stats, utils, XML
Depends: R (>= 4.0.0), shiny (>= 1.2)
Suggests: covr, devtools, emoji, knitr, pkgdown, rmarkdown, roxygen2,
        testthat
RoxygenNote: 7.2.3
VignetteBuilder: knitr
URL: https://ahgroup.github.io/DSAIRM/,
        https://github.com/ahgroup/DSAIRM/
BugReports: https://github.com/ahgroup/DSAIRM/issues
NeedsCompilation: no
Packaged: 2023-08-23 18:21:53 UTC; Andreas
Author: Andreas Handel [aut, cre] (<https://orcid.org/0000-0002-4622-1146>),
  Cody Dailey [ctb],
  Yang Ge [ctb],
  Spencer Hall [ctb],
  Brian McKay [ctb],
  Sina Solaimanpour [ctb],
  Alexis Vittengl [ctb],
  Henok Woldu [ctb]
Repository: CRAN
Date/Publication: 2023-08-23 18:50:05 UTC
Built: R 4.6.0; ; 2026-04-25 22:35:32 UTC; unix
