# Generated by roxygen2: do not edit by hand

export(check_simresults)
export(dsairmmenu)
export(generate_documentation)
export(generate_fctcall)
export(generate_ggplot)
export(generate_modelsettings)
export(generate_plotly)
export(generate_shinyinput)
export(generate_text)
export(run_model)
export(simulate_acutevirusir_ode)
export(simulate_bacteria_fit)
export(simulate_basicbacteria_discrete)
export(simulate_basicbacteria_modelexploration)
export(simulate_basicbacteria_ode)
export(simulate_basicvirus_fit)
export(simulate_basicvirus_modelexploration)
export(simulate_basicvirus_ode)
export(simulate_basicvirus_stochastic)
export(simulate_chronicvirusir_ode)
export(simulate_confint_fit)
export(simulate_drugresistance_stochastic)
export(simulate_extendedbacteria_ode)
export(simulate_fludrug_fit)
export(simulate_modelcomparison_fit)
export(simulate_modelvariants_ode)
export(simulate_pkpdmodel_ode)
export(simulate_usanalysis)
export(simulate_virusandir_ode)
export(simulate_virusandtx_ode)
import(ggplot2, except = last_plot)
import(plotly)
import(shiny)
importFrom(dplyr,filter)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(gridExtra,grid.arrange)
importFrom(nloptr,nloptr)
importFrom(rlang,.data)
importFrom(stats,median)
importFrom(stats,reshape)
importFrom(utils,head)
importFrom(utils,read.csv)
importFrom(utils,tail)
